
#ifndef PLS_ABSTRACT_TASK_H
#define PLS_ABSTRACT_TASK_H

#define PLS_UNIQUE_ID __COUNTER__

#include "pls/internal/base/spin_lock.h"

namespace pls {
    namespace internal {
        namespace scheduling {
            class abstract_task {
                int depth_;
                int unique_id_;
                abstract_task* child_task_;
                base::spin_lock spin_lock_;

            public:
                explicit abstract_task(int depth, int unique_id):
                    depth_{depth},
                    unique_id_{unique_id},
                    child_task_{nullptr},
                    spin_lock_{} {};

                virtual void execute() = 0;
                const base::spin_lock& spin_lock() { return spin_lock_; }

            protected:
                virtual bool my_stealing(abstract_task *other_task) = 0;

                bool steal_work() {
                    // get scheduler
                    // select victim
                    // try steal
                    // |-- see if same depth is available
                    // |-- see if equals depth + id
                    // |-- try user steal if matches (will return itself if it could steal)
                    // |-- try internal steal if deeper tasks are available
                    // |-- if internal steal worked, execute it
                    // return if the user steal was a success
                    return false;
                };
            };
        }
    }
}

#endif //PLS_ABSTRACT_TASK_H
